import Word from "./enums/reserved";

export default class Parser {
  static resolve() {
    return Word.reserved;
  }

  static codeToHtml(code) {
    Word.reserved.forEach((word) => {
      code = this.replaceWord(code, word.toLowerCase());
      code = this.replaceWord(code, word.toUpperCase());
    });
    return code;
  }

  static fastActions(fullCode, code, fastActions) {
    code = this.clearCode(code);
    //fastActions
    const command = code.split(" ")[code.split(" ").length - 1];
    if (fastActions.filter(a => a.key === command).length > 0) {
      return {
        mode: "custom_command",
        object: command,
        alias: null
      }
    }
    //expand all columns
    if (code[code.length - 1] === "*") {
      const symbols = code.split('.');
      if (symbols.length > 1) {
        //last symbol is *
        if (symbols[symbols.length - 1] === "*") {
          const words = symbols[symbols.length - 2].split(" ");
          const table = this.usedAliases(fullCode, words[words.length - 1]);
          const result = {
            mode: "all_columns",
            object: table,
            alias: words[words.length - 1]
          };
          console.log(result);
          return result;
        }
      }
    }

    return {
      mode: "none",
      object: undefined,
      alias: undefined
    }
  }

  static replaceWord(code, word) {
    let replace = "\\b" + word + "\\b";
    let re = new RegExp(replace);
    code = code
      .split(re)
      .join("<span class='mparse-reserved'>" + word + "</span>");
    return code;
  }

  static getMode(code, lastWord, result) {
    if (lastWord === "from") {
      result.mode = "tables";
      result.enabled = true;
    }
    if (lastWord === "select") {
      result.mode = "aliases";
      result.enabled = true;
    }
    if (lastWord === "where") {
      result.mode = "aliases";
      result.enabled = true;
    }
    if (lastWord === "join") {
      result.mode = "join";
      result.enabled = true;
    }
    if (lastWord === "use") {
      result.mode = "databases";
      result.enabled = true;
    }
    if (lastWord === "on") {
      result.mode = "join_details";
      result.enabled = true;
    }
    if (lastWord[lastWord.length - 1] === ".") {
      result.mode = "columns";
      result.enabled = true;
      console.log("word 1: " + lastWord.substring(0, lastWord.length - 1));
      result.table = this.usedAliases(code, lastWord.substring(0, lastWord.length - 1));
    }

    if (lastWord.split(".").length > 1) {
      result.enabled = true;
      result.mode = "columns"
      result.filtered = true;
      result.filter = lastWord.split(".")[1];
      console.log("word 2: " + lastWord.split(".")[0]);
      result.table = this.usedAliases(code, lastWord.split(".")[0]);
    }

    return result;
  }

  static mode(fullCode, code) {
    // console.log(code);
    let result = {
      enabled: false,
      mode: "none",
      filtered: false,
      filter: "",
    };
    code = this.clearCode(code).toLowerCase();
    let words = code.split(" ");
    if (words.length > 0) {
      let lastWord = words.pop();
      if (lastWord === "") {
        result.enabled = true;
        lastWord = words.pop();
      }
      if (lastWord) {
        result = this.getMode(fullCode, lastWord, result);
        if (result.enabled) {
          return result;
        }
        result.filter = lastWord;
        result.filtered = true;
        lastWord = words.pop();
        if (lastWord) {
          result = this.getMode(fullCode, lastWord, result);
          return result;
        }
      }
    }
    return result;
  }

  static clearCode(code) {
    let clear = code.replace(new RegExp(/\t/g), " ");
    clear = clear.replace(new RegExp(/\n/g), " ");
    while (clear.indexOf("  ") > -1) clear = clear.split("  ").join(" ");
    return clear;
  }

  static genereateAlias(name) {
    let alias = "";
    let localName = name.slice();
    while (localName.search(/[A-Z]/) > -1) {
      const index = localName.search(/[A-Z]/);
      alias += localName[index];
      localName = localName.slice(0, index) + localName.slice(index + 1);
    }
    if (alias.length > 0) return alias.toLocaleLowerCase();
    return name.substring(0, 1).toLowerCase();
  }

  static getAlias(code, name, position) {
    let nextCode = code.substring(position, code.length);

    if (nextCode.toLowerCase().split(" ").filter(e => e !== "")[0] === "as")
      return "";
    const aliases = this.usedAliases(code, null);
    name = this.genereateAlias(name);
    let index = 0;
    while (aliases.indexOf(name) > -1) {
      if (index > 0) {
        name = name.substring(0, name.length - String(index).length);
      }
      index += 1;
      name += String(index);
    }
    return name;
  }

  static usedAliases(code, alias) {
    let clear = this.clearCode(code);
    let result = [];
    const words = clear.split(" ");
    const asses = words.filter((e) => e.toLowerCase() === "as");

    while (words.findIndex((e) => e.toLowerCase() === "as") > -1) {
      const index = words.findIndex((e) => e.toLowerCase() === "as");
      if (!alias) {
        result.push(words[index + 1]);
      } else {
        if (words[index + 1] === alias) {
          return words[index - 1];
        }
      }
      words.splice(index, 2);
    }
    if (!alias) {
      return result;
    } else {
      return null;
    }
  }

  static intellisenseData(items, filter) {
    let result = undefined;
    if (filter) {
      result = [];
      result = items.filter(
        (i) => i.name.toLowerCase().indexOf(filter.toLowerCase()) > -1
      );
      result = result
        .concat(
          items.filter(
            (i) =>
              i.name.match(
                new RegExp(
                  filter
                    .split("*")
                    .join("")
                    .split("")
                    .join("\\w{0,}")
                , "ig")
              )
          )
        )
        .unique();
    }
    if (!result) return items;
    else return result;
  }
}
